package com.muyu.common.cache.abs;


import com.muyu.common.cache.BasePool;


import com.muyu.common.cache.MysqlPoolConfig;
import com.muyu.common.cache.data.MysqlPrefix;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Mysql的连接池信息
 *
 * @author AnNan.Wang
 * @Date 2024/4/29 029 14:36
 */

@Component
public class MysqlPool implements BasePool<Connection> {

    /**
     * 打印日志
     */
    private static  final  Logger log = LoggerFactory.getLogger(MysqlPool.class);

    /**
     * 等待队列 基础队列
     */
    private Queue<Connection> mysqlConnQueue = null;

    /**
     * 活动队列
     */
    private Queue<Connection> activeMysqlQueue = null;

    /**
     * 记录队列的连接总数
     */
    private AtomicInteger count = new AtomicInteger();

    /**
     * 连接参数
     */
    @Autowired
    public MysqlPoolConfig mysqlPoolConfig;



    /**
     * 进行实例化连接池 并且加载驱动
     * @param mysqlPoolConfig
     */
    public MysqlPool(MysqlPoolConfig mysqlPoolConfig) {
        log.info("Mysql连接池实例化完成");
        this.mysqlPoolConfig = mysqlPoolConfig;
        MysqlPrefix.driver(this.mysqlPoolConfig.getDriverName());
    }

    @Override
    public void init() {
        //初始话连接数
        int initTotal = this.mysqlPoolConfig.getInitTotal();
        //最大连接数
        int maxTotal = this.mysqlPoolConfig.getMaxTotal();

        this.mysqlConnQueue = new LinkedBlockingQueue<>();
        this.activeMysqlQueue = new LinkedBlockingQueue<>();

        for (int i = 0; i < initTotal; i++) {
            this.mysqlConnQueue.offer(creatConnection());
            count.incrementAndGet();
        }
        log.info("数据库连接池初始化完成");
    }

    @Override
    public Connection getConn() {
        long startTime = System.currentTimeMillis();

        Connection conn = this.mysqlConnQueue.poll();

        if (conn == null) {
            this.activeMysqlQueue.offer(conn);
            return conn;
        }
        if (count.get()<this.mysqlPoolConfig.getMaxTotal()) {
            Connection connection = creatConnection();
            this.activeMysqlQueue.offer(connection);
            count.incrementAndGet();
            return connection;
        }
        if (System.currentTimeMillis() - startTime > this.mysqlPoolConfig.getMaxWaitTimes()) {
            throw new IllegalStateException("连接超时");
        }
        return null;
    }

    /**
     * 归还连接
     * @param conn
     */
    @Override
    public void reaplase(Connection conn) {
        log.info("删除活动队列当中的连接");
        if (this.activeMysqlQueue.remove(conn)) {
            log.info("把这个队列放到空闲队列当中");
            this.mysqlConnQueue.offer(conn);
        }
    }

    /**
     * 连接数据库信息
     * @return
     */
    @Override
    public Connection creatConnection() {
        String url = null;

        if (this.mysqlPoolConfig.getDataAccessTypeId() == 3) {
            url = this.mysqlPoolConfig.getUrl();
        }else if (this.mysqlPoolConfig.getDataAccessTypeId() == 5){
            url = this.mysqlPoolConfig.getPostgreSqlUrl();
        }

        String userName = this.mysqlPoolConfig.getUserName();
        String password = this.mysqlPoolConfig.getPassword();

        Connection connection = null;

        try {
            connection = DriverManager.getConnection(url, userName, password);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        log.info("初始化了一个数据库连接：{ip: " + this.mysqlPoolConfig.getIp() + " port: " + this.mysqlPoolConfig.getPort() + " databaseName: " + this.mysqlPoolConfig.getDatabaseName() + "}");
        return connection;
    }

    /**
     * 关闭连接
     */
    @Override
    public void closeConn() {
        closeBaseConn();
        closeActiveConn();
    }

    private void closeActiveConn() {
        Connection poll = this.mysqlConnQueue.poll();

        if (poll != null) {
            try {
                poll.close();
            } catch (SQLException e) {
                e.printStackTrace();
                try {
                    if(!poll.isClosed()) {
                       this.mysqlConnQueue.offer(poll);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            } finally {
                closeBaseConn();
            }
        }
    }

    private void closeBaseConn() {
        Connection poll = this.mysqlConnQueue.poll();

        if (poll != null) {
            try {
                poll.close();
            } catch (SQLException e) {
                e.printStackTrace();
                try {
                    if(!poll.isClosed()) {
                        this.mysqlConnQueue.offer(poll);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            } finally {
                closeBaseConn();
            }
        }
    }
}
