package com.muyu.data.source.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.data.source.domain.*;
import com.muyu.data.source.domain.model.DataTableValue;
import com.muyu.data.source.domain.model.DatabaseTableModel;
import com.muyu.data.source.domain.red.DataSourceTableRed;
import com.muyu.data.source.domain.req.DatabaseConnect;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.data.source.domain.req.DataSourceQueryReq;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import com.muyu.data.source.domain.req.DataSourceEditReq;
import com.muyu.data.source.service.DataSourceService;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 数据源Controller
 * @author AnNan.Wang
 * @date 2024-04-21
 */
@Api(tags = "数据源")
@RestController
@RequestMapping("/source")
public class DataSourceController extends BaseController {
    @Autowired
    private DataSourceService dataSourceService;

    /**
     * 查询数据源列表
     */
    @ApiOperation("获取数据源列表")
    @PostMapping("/list")
    public Result<TableDataInfo<DataSource>> list(DataSourceQueryReq dataSourceQueryReq) {
        startPage();
        List<DataSource> list = dataSourceService.list(DataSource.queryBuild(dataSourceQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出数据源列表
     */
    @ApiOperation("导出数据源列表")
    @RequiresPermissions("source:source:export")
    @Log(title = "数据源", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataSource dataSource) {
        List<DataSource> list = dataSourceService.list(dataSource);
        ExcelUtil<DataSource> util = new ExcelUtil<DataSource>(DataSource.class);
        util.exportExcel(response, list, "数据源数据");
    }

    /**
     * 获取数据源详细信息
     */
    @ApiOperation("获取数据源详细信息")
    @RequiresPermissions("source:source:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<DataSource> getInfo(@PathVariable("id") Long id) {
        return Result.success(dataSourceService.getById(id));
    }

    /**
     * 新增数据源
     */
    @RequiresPermissions("source:source:add")
    @Log(title = "数据源", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增数据源")
    public Result<String> add(@RequestBody DataSourceSaveReq dataSourceSaveReq) {
        return toAjax(dataSourceService.save(DataSource.saveBuild(dataSourceSaveReq)));
    }

    /**
     * 修改数据源
     */
    @RequiresPermissions("source:source:edit")
    @Log(title = "数据源", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改数据源")
    public Result<String> edit(@PathVariable Long id, @RequestBody DataSourceEditReq dataSourceEditReq) {
        return toAjax(dataSourceService.updateById(DataSource.editBuild(id,dataSourceEditReq)));
    }

    /**
     * 删除数据源
     */
    @RequiresPermissions("source:source:remove")
    @Log(title = "数据源", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除数据源")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(dataSourceService.removeBatchByIds(ids));
    }
    /**
     * 查询接入类型列表
     */
    @GetMapping("/findAccessType")
    public Result<List<AccessType>> findAccessType(){
        List<AccessType> accessTypeList=dataSourceService.findAccessType();
        return Result.success(accessTypeList);
    }

    /** 测试连接 */
    @PostMapping("/testConnection")
    public Result testConnection(@RequestBody DataSourceSaveReq dataSourceSaveReq){
        return dataSourceService.testConnection(dataSourceSaveReq);
    }

    /**
     * 同步
     * @param dataSource
     * @return
     */
    @PostMapping("/synchronization")
    public Result synchronization(@RequestBody DataSource dataSource){
        return dataSourceService.synchronization(dataSource);
    }

    //查询结构
    @PostMapping("/findDataBaseTable")
    public Result<List<DatabaseTableModel>> findDataBaseTable(@RequestBody DatabaseConnect databaseConnect){
        return dataSourceService.findDataBaseTable(databaseConnect);
    }


    //查询接入源信息
    @GetMapping("/findStructure")
    public Result<Structure> findStructure(){
        return dataSourceService.findStructure();
    }

    //点击查询表
    @PostMapping("/structureList")
    public Result<Structure> structureList(@RequestParam("id") Integer id){
        return dataSourceService.structureList(id);
    }

    //根据表名查询详细信息
    @PostMapping("/database")
    public Result<List<DatabaseTableModel>> database (@RequestParam("name") String name){
        return dataSourceService.database(name);
    }

    @PostMapping("/table")
    public Result<List<DatabaseTableInformation>> table (@RequestParam("databaseName") String databaseName){
        return dataSourceService.table(databaseName);
    }

    //统计数量
    @GetMapping("/quantity")
    public Result<List<DatabaseTableInformation>> quantity(){
        return dataSourceService.quantity();
    }


    @GetMapping("/information")
    public Result<List<DataSourceTableRed>> information(){
        return dataSourceService.information();
    }

}
